%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compile code
!make

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run different algorithms
%name = './tsukuba288x384';
inputdir = './data/';
outputdir = './results';
name = 'pascal2008_000938';
inputpath = [inputdir '/' name '.txt'];

alglist = '-a -1 -a 2 -a 3'; % -a -2 will run MPLP but that's pretty slow

%exec = './dtbca_potts';
exec = './dtbca_gen';

% careful, this will take a long time
%MAXITER =500;
MAXITER = 5;
system(sprintf('%s %s -i %s -o %s --MAXITER %d',exec, alglist, inputpath, outputdir, MAXITER));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters
alg_list = [-1 2 3];
lnames = {'MPLP', 'TRW-S', ...
    'Stars-TBCA', 'Combs-TBCA', 'LPDG-DTBCA', 'WTA-DTBCA'};
plot_primal = 0;
do_subplot = 1;
%ss = 30; % subsample
ss = 1; % subsample

lnames = lnames(alg_list+3);

colours = [0.5 0.5 0.5; % MPLP
           0 0.7 0;       % TRW-S
            0.7 0 0.7;    % Stars
            0 1 1;      % Combs
            0 0 1;      % LPDG
            1 0 0];     % WTA
symbols = {'*','x','o','>','<','^','>','<'};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%load data
savedir = fullfile(outputdir,name);

for k=alg_list
    enhist{k+3} = load(sprintf('%s/enhist_alg_t%d.txt',savedir,k));
    lbhist{k+3} = load(sprintf('%s/lbhist_alg_t%d.txt',savedir,k));
    timehist{k+3} = load(sprintf('%s/timehist_alg_t%d.txt',savedir,k));
    %edgecnthist{k+3} = load(sprintf('%s/edgecnthist_alg_t%d.txt',savedir,k));
    %tbhist{k+3} = load(sprintf('%s/tbhist_alg_t%d.txt',savedir,k));
    disthist{k+3} = load(sprintf('%s/disthist_alg_t%d.txt',savedir,k));
    if k==-2 %MPLP maximizes instead of minimizing
        enhist{k+3} = -enhist{k+3};
        lbhist{k+3} = -lbhist{k+3};
    end
    
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make Plots
if do_subplot
    figure; set(gcf, 'position', [150   375   982   366]);
end

% lb/en-vs-time
if do_subplot
    subplot(1,2,1); 
else
    figure; set(gcf,'position',[132   195   703   573]);
end
hold on;
for k=alg_list
    tt = cumsum(timehist{k+3});
    plot(tt(1:ss:end),lbhist{k+3}(1:ss:end),'color',colours(k+3,:), 'marker',symbols{k+3}, 'markersize',10);
end
if plot_primal
for k=alg_list
    tt = cumsum(timehist{k+3});
    plot(tt(1:ss:end),enhist{k+3}(1:ss:end),'--','color',colours(k+3,:), 'marker',symbols{k+3});
end
end
xlabel('Time (sec)');
ylabel('Dual Objective');

% plot MAP
map = 0.5*(lbhist{2}(end)+enhist{2}(end));
tt = cumsum(timehist{alg_list(1)+3});
%tt = 400;
xrange = [-10 tt(end)];
plot(xrange, map*ones(1,2), 'k-.');
legend([lnames 'MAP (Just a guess)'],'location','best')
%grid on;

% lb-vs-dist-trans
if do_subplot
    subplot(1,2,2); 
else
    figure; set(gcf,'position',[132   195   703   573]);
end
hold on;
for k=alg_list
    dt = cumsum(disthist{k+3});
    plot(dt(1:ss:end),lbhist{k+3}(1:ss:end),'color',colours(k+3,:), 'marker',symbols{k+3}, 'markersize',10);
end
if plot_primal
for k=alg_list
    dt = cumsum(disthist{k+3});
    plot(dt(1:ss:end),enhist{k+3}(1:ss:end),'--','color',colours(k+3,:), 'marker',symbols{k+3});
end
end
xlabel('# Distance Transforms');
ylabel('Dual Objective');
% plot MAP
plot(0.5*(lbhist{2}(end)+enhist{2}(end))*ones(1,length(lbhist{2})), 'k-.');
legend([lnames 'MAP (Just a guess)'],'location','best')
%grid on;
% plot MAP
map = 0.5*(lbhist{2}(end)+enhist{2}(end));
tt = cumsum(disthist{2});
xrange = [-30 tt(end)];
tt = 400;
plot(xrange, map*ones(1,2), 'k-.');
legend([lnames 'MAP (Just a guess)'],'location','best')
%grid on;


