//
//  mrf_energy.h
//  dtbca_public_release
//
//  Created by Dhruv Batra on 6/11/12.
//  Copyright 2012 Dhruv Batra. All rights reserved.
//
//  Inspired by Vladimir Kolmogorov's MRFEnergy.h in TRW-S

#ifndef __MRF_ENERGY_H__
#define __MRF_ENERGY_H__


// Abstract Base Class that cannot be instantiated, 
// but describes the basic elements of a MRF Energy Class
class PairwiseEnergy 
{
public:
    /////////////////////////////////////////////////////////
    // Data
    typedef int Labeltype;
    typedef int Inttype;
    
    // No. of nodes, edges 
    int nnodes; 
    int nedges;
    
    int nrows, ncols; // Not always used

    Labeltype sum_of_cardinalities;

    // List of (i,j) pairs (0-based numbering)
    int *edgelist;
    
    // Node and edge potentials
    double *D;
    double *V;
    
    double DV_maxval, DV_minval;
    
    /////////////////////////////////////////////////////////
    // Functions
    
    // Constructor
    PairwiseEnergy():
    nnodes(0), nedges(0), 
    nrows(0), ncols(0),
    sum_of_cardinalities(0),
    edgelist(NULL), D(NULL), V(NULL), 
    DV_maxval(-INF), DV_minval(INF)
    {}
    
    // Destructor
    ~PairwiseEnergy()
    {
        if (edgelist)
            delete [] edgelist;
        if (D)
            delete [] D;
        if (V)
            delete [] V;
    }
    
    // Pure Virtual Functions that must be defined by derived classes
    virtual void LoadEnergyFromFile(char *fname) =0;
    virtual void RoundEnergy() =0;

};

#endif