//
//  typeUAI.cpp
//  dtbca_public_release
//
//  Created by Dhruv Batra on 6/11/12.
//  Copyright 2012 Dhruv Batra. All rights reserved.
//

#include "typeUAI.h"

/////////////////////////////////////////////////////////
void TypeUAI::LoadEnergyFromFile(char *fname)
{
    // Open file
    string input_file(fname);
    ifstream in(input_file.c_str());
    
    CHECK(in.good(),StringPrintf("Could not open file: %s",fname));
    
    // Read type of network
    string name; 
    
    in >> name; 
    CHECK(name.compare("MARKOV")==0, "Only Markov networks are supported. Are you sure this is a typeUAI energy file?");
    
    
    // Read size of graph
    in >> nnodes;
    CHECK(nnodes>0, "No. of nodes can't be negative. Are you sure this is a typeUAI energy file?");
    
    
    vector<int> edgelistvec;    
    vector<Labeltype> card_cumulsum;
    vector<Labeltype> edgepotsize_cumulsum;
    
    
    // Read node cardinalities
    cardinalities = new Labeltype[nnodes];
    card_cumulsum.resize(nnodes+1); card_cumulsum[0] = 0;
    Labeltype cardinality_i; sum_of_cardinalities = 0;
    for (int i = 0; i < nnodes; i++) 
    {
        in >> cardinality_i;
        cardinalities[i] = cardinality_i;
        sum_of_cardinalities += cardinality_i;
        card_cumulsum[i+1] = card_cumulsum[i] + cardinality_i;
        
        //cout << cardinalities[i] << " ";
        CHECK(in.good(), "Could not finish reading cardinalities. Are you sure this is a typeUAI energy file?");
    }
    
    // Read no. of factors
    in >> nfactors;
    
    factor_size.resize(nfactors); factor_id.resize(nfactors);
    int temp1, temp2;
    edgepotsize_cumulsum.push_back(0);

    // Loop and read factor members
    for (int i=0; i!=nfactors; ++i) 
    {
        in >> temp1;
        factor_size[i] = temp1; 
        
        in >> temp1;
        if (factor_size[i] == 1) // this is a node factor
            factor_id[i] = temp1;
        else if (factor_size[i] == 2) // this is an edge factor
        {
            edgelistvec.push_back(temp1);
            in >> temp2;
            edgelistvec.push_back(temp2);
            
            factor_id[i] = nedges;
            ++nedges; 
            
            sum_of_edge_potential_sizes += 
            cardinalities[temp1] * cardinalities[temp2];
            edgepotsize_cumulsum.push_back(sum_of_edge_potential_sizes);
        }
        else
            CHECK(0, "Only node & edge factors are supported. File contains something different.");

        CHECK(in.good(), "Could not finish reading cardinalities. Are you sure this is a typeUAI energy file?");
    }
    
    
    // Allocate Space
    edgelist  = new int [nedges*2];
    D = new double[sum_of_cardinalities];
    V = new double[sum_of_edge_potential_sizes];
    
    
    // Now read factor potentials
    int offset;
    for (int i=0; i!=nfactors; ++i) 
    {
        int cardprod; double potential_value, energy;
        in >> cardprod;
        
        if (factor_size[i] == 1) // node potential
        {
            CHECK_EQ(cardprod, cardinalities[factor_id[i]], "Incorrectly sized factor");
          
            offset = card_cumulsum[factor_id[i]];
            
            // Read node potential
            for (int k = 0; k != cardprod; ++k) 
            {
                in >> potential_value;
                energy = Potential2Energy(potential_value);
                D[offset++] = energy;
                //cout << D[offset-1] << " ";
                
                // keep track of max entry
                if (energy > DV_maxval)
                    DV_maxval = energy;
                // keep track of min entry
                if (energy < DV_minval)
                    DV_minval = energy;
            }
        } // Edge of node potential
        
        else if (factor_size[i] == 2) // edge potential
        {
            int e = factor_id[i];
            int n0, n1;
            // UAI files store factors in row-major format. Our code wants them in column-major. 
            // Just flip the order of variables for a pairwise factor and done. 
            // n0 = edgelistvec[2*e]; n1 = edgelistvec[2*e+1];
            n1 = edgelistvec[2*e]; n0 = edgelistvec[2*e+1];
            
            Labeltype card0, card1;
            card0 = cardinalities[n0];
            card1 = cardinalities[n1];

            CHECK_EQ(cardprod, card0*card1, "Incorrectly sized factor");

            // Copy edgelist
            edgelist[2*e] = n0;
            edgelist[2*e+1] = n1;
            
            offset = edgepotsize_cumulsum[e];
            // Read edge potential
            for (int ij = 0; ij != card0 * card1; ++ij)
            {
                in >> potential_value;
                energy = Potential2Energy(potential_value);
                V[offset++] = energy;
                //cout << V[offset - 1] << " ";
                
                // keep track of max entry
                if (energy > DV_maxval)
                    DV_maxval = energy;
                // keep track of min entry
                if (energy < DV_minval)
                    DV_minval = energy;
            }
        } // End of edge potential
        
        CHECK(in.good(), "Could not finish reading factor tables. Are you sure this is a typeUAI energy file?");
    } // End of reading factors   
    
    
    // close file
    in.close();
}

/////////////////////////////////////////////////////////
void TypeUAI::RoundEnergy()
{
    // Scale to Inttype
    
    // Node energies
    int offset = 0;
    for (int i=0; i!=nnodes; ++i) 
        for (int l=0; l!=cardinalities[i]; ++l) 
        {
            D[offset] = (Inttype) ((D[offset] - DV_minval) * MAX_INT/DV_maxval);
            ++offset;
        }
    // Edge energies
    for (int e=0; e != nedges; ++e) 
    {
        V[e] = (Inttype) ((V[e] - DV_minval) * MAX_INT/DV_maxval);
    }
}
