//
//  typeUAI.h
//  dtbca_public_release
//
//  Created by Dhruv Batra on 6/11/12.
//  Copyright 2012 Dhruv Batra. All rights reserved.
//

#ifndef __TYPE_UAI_H__
#define __TYPE_UAI_H__

#include <iostream>
#include <fstream>

#include "utils.h"
#include "constants_declaration.h"
#include "mrf_energy.h"

// define log(0) (required for converting potentials to energies)
const int LOGZERO = -1000; 

// Class that holds Data (graph, node and edge potentials) for a General MRF
// in UAI file format ( http://www.cs.huji.ac.il/project/PASCAL/fileFormat.php )
// The only restriction is that factors cannot be larger than order 2 (no triplet cliques or higher)
class TypeUAI : public PairwiseEnergy
{
public:
    /////////////////////////////////////////////////////////
    // Data
    int nfactors;
    vector<int> factor_size;
    vector<int> factor_id;
    
    Labeltype* cardinalities;
    Labeltype sum_of_edge_potential_sizes;
    
    /////////////////////////////////////////////////////////
    // Functions
    
    // Constructor
    TypeUAI():
    nfactors(0),
    cardinalities(NULL),
    sum_of_edge_potential_sizes(0)
    { }
    
    // Destructor
    ~TypeUAI()
    {    
        if (cardinalities)
            delete [] cardinalities;
    }
    
    
    void LoadEnergyFromFile(char *fname);
    void RoundEnergy();
    
    // UAI files contain potentials. Convert them to energies.
    double Potential2Energy(double potential)
    {
        if (fabs(potential) < 1e-20)
            return -LOGZERO;
        else
            return -log10(potential);
    }

};



#endif