/*
 *  mplp_main_seg.cpp
 *  dtrw
 *
 *  Created by Dhruv Batra on 3/09/11.
 *  Copyright 2011 Dhruv Batra. All rights reserved.
 *
 */

#include <iostream>

using namespace std;
#include <vector>
#include "mplp_alg.h"

#include <string>
#include <fstream>

//void testMPLPGridPotts(int sizeX, int sizeY, int _numpairs, int K, double *D, double *V)
int main( int argc, char *argv[] )
{
        ////////////////////////////////////
        // read file
        //string input_file = "/home/dbatra/research/ttic_projects/v2_dtrw/data_seg/2008_000011.txt"; 
        string input_file = "/home/dbatra/research/ttic_projects/v2_dtrw/data_seg/2008_000938.txt"; 
	
	FILE *fp = fopen( input_file.c_str(), "rt" );
	if (!fp)
	  {
	    printf("Could not open file. Quitting!\n");
	    exit(-1);
	  }
	      
	  
	int _numpoints=0, _numlabels=0,_numpairs=0;

	// Read num_nodes num_labels num_edges
	fscanf(fp, "%d %d %d", &_numpoints, &_numlabels, &_numpairs);
	
	// hard-coded stuff for this example
	int sizeX = 500;
	int sizeY = 375;
	// HACKHACK, text file contains 8-connected edges, so only consider 4-connected subset
	_numpairs = 2*sizeX*sizeY - sizeX - sizeY;

	
	// Initialize space for node and edge energies
	double *D = new double[_numpoints*_numlabels];
	double *V = new double[_numpairs]; // Contrast-Sensitive Potts
	
	// Read node energies
	double tmp2;
	for (int i=0; i!=_numpoints; ++i) {
	  //cout << i << ":\t";
	  for (int l=0; l!=_numlabels; ++l) 
	    {
	      //fscanf(fp, "%lf ",&tmp2);
	      fscanf(fp, "%lf ",&tmp2);
	      D[i*_numlabels + l] = tmp2;
	    }
	  //cout << endl;
	}
	
	
	// Read edge energies
	int n0;
	int n1;
	for (int e=0; e != _numpairs; ++e) {
	  //cout << n0 << "--" << n1 << "\t" << tmp2 << endl;
	  fscanf(fp, "%d %d %lf", &n0, &n1, &tmp2);
	  V[e] = tmp2;
	}
	
	
	////////////////////////////////////
	// parameters for MPLP
	int MAXITER = 100;
	double obj_del_thr = 1e-5;
	double int_gap_thr = 1e-5;

	
	////////////////////////////////////
	// prepare input for mplp
	int n_nodes = sizeX*sizeY;
	vector<int> n_labels(n_nodes, _numlabels);
	vector<double> potts_lambda(V, V + _numpairs); 
	
	////////////////////////////////////
	printf("nIters=%d\nobj_del_thr=%lg\nint_gap_thr=%lg\n",MAXITER,obj_del_thr,int_gap_thr);
	
	// initialize using a constructor specially written for Grids with Potts model
	MPLPAlg mplp(sizeY, sizeX, n_labels, potts_lambda, D); 
	
	// Run message passing
	mplp.RunMPLP(MAXITER,obj_del_thr,int_gap_thr);
	
	////////////////////////////////////
	// Save data to files
	string lbn = "lbhist", enn = "enhist", tn = "timehist";
	
	mplp.Write(NULL,NULL,NULL,(char*) lbn.c_str(),(char*) enn.c_str(),(char*) tn.c_str());
}
